@@ -15,8 +15,7 @@ if ((!(-e $dest)) or (!(-d $dest))) {
 }
 
 # Delete any existing version file
-if (-f ver.cfg)
-{
+if (-f ver.cfg) {
     unlink(ver.cfg);
 }
 # Get the latest version file from the repository
@@ -42,7 +41,7 @@ $svnver = $1;
 # Create the version suffix
 $qver = "$qver.$svnver";
 
-system("powershell Remove-Item -Recurse -Force qgvdial*");
+system("powershell Remove-Item -Recurse -Force qgvdial-*");
 
 $cmd = "svn export $repo qgvdial-$qver";
 print("$cmd\n");
@@ -58,11 +57,16 @@ $cmd = "cd qgvdial-$qver & perl ./build-files/version.pl __QGVDIAL_VERSION__ $qv
 print("$cmd\n");
 system($cmd);
 
-# # Fix the QML files
+# Mosquitto is merged straight into the build
 $cmd = "cd qgvdial-$qver/src & echo something>mqlib-build";
 print "$cmd\n";
 system($cmd);
 
+# Symbian 3 (to including Belle)  should not have qt-components
+$cmd = "cd qgvdial-$qver/src & touch no-qt-components";
+print "$cmd\n";
+system($cmd);
+
 # This is the way to enter a directory and setup the remainder variables
 my $enterdir = "cd qgvdial-$qver/src & set BUILDDIR=%CD% & set SOURCEDIR=%CD%";
 
