@@ -31,6 +31,9 @@ MainWindow::setStatus(const QString &strText, int /*timeout = 3000*/)
 void
 MainWindow::init()
 {
+    connect(&gvApi, SIGNAL(twoStepAuthentication(AsyncTaskToken *)),
+             this , SLOT(onTwoStepAuthentication(AsyncTaskToken *)));
+
     // Status from contacts object
     QObject::connect (&oContacts, SIGNAL (status   (const QString &, int)),
                        this     , SLOT   (setStatus(const QString &, int)));
@@ -96,7 +99,7 @@ MainWindow::doWork ()
 bool
 MainWindow::checkParams ()
 {
-    bool rv = false, bUseDefaultIni = false;
+    bool rv = false, bUseDefaultIni = false, tempBool;
     QStringList args = qApp->arguments ();
     if (args.length () < 2) {
         qWarning ("No ini file specified, using default");
@@ -146,6 +149,29 @@ MainWindow::checkParams ()
             strPass = byD;
         }
 
+        if (!settings.contains ("tfaRequired")) {
+            tempBool = false;
+            settings.setValue ("tfaRequired", tempBool);
+            tfaRequired = false;
+        } else {
+            tfaRequired = settings.value("tfaRequired").toBool();
+        }
+
+        if (tfaRequired) {
+            if (!settings.contains ("cpass")) {
+                qWarning ("Ini file does not contain an application specific "
+                          "password");
+                cout << "Enter application specific password:";
+                in >> strCPass;
+                cipher (strCPass.toLocal8Bit (), byD, true);
+                settings.setValue ("cpass", QString(byD.toHex ()));
+            } else {
+                byD = settings.value("cpass").toByteArray();
+                cipher (QByteArray::fromHex (byD), byD, false);
+                strCPass = byD;
+            }
+        }
+
         checkTimeout = 0;
         if (settings.contains ("check_timeout")) {
             checkTimeout = settings.value ("check_timeout").toUInt ();
@@ -328,6 +354,8 @@ MainWindow::doLogin ()
             break;
         }
 
+        //loadCookies ();
+
         bOk = connect(token, SIGNAL(completed(AsyncTaskToken*)),
                       this , SLOT(loginCompleted(AsyncTaskToken*)));
 
@@ -373,7 +401,29 @@ MainWindow::loginCompleted (AsyncTaskToken *token)
 
         bIsLoggedIn = true;
 
-        oContacts.setUserPass (strUser, strPass);
+        if (tfaRequired) {
+            Q_DEBUG("TFA");
+
+            if (strCPass.isEmpty ()) {
+                QTextStream in(stdin);
+
+                cout << "Enter application specific password:";
+                in >> strCPass;
+            }
+
+            QByteArray byD;
+            QSettings settings (strIni, QSettings::IniFormat, this);
+
+            cipher (strCPass.toLocal8Bit (), byD, true);
+            settings.setValue ("cpass", QString(byD.toHex ()));
+            settings.setValue ("tfaRequired", tfaRequired);
+        } else {
+            strCPass = strPass;
+        }
+
+        //saveCookies ();
+
+        oContacts.setUserPass (strUser, strCPass);
         oContacts.loginSuccess ();
         oInbox.loginSuccess ();
 
@@ -412,6 +462,24 @@ MainWindow::logoutCompleted (AsyncTaskToken *token)
     }
 }//MainWindow::logoutCompleted
 
+void
+MainWindow::onTwoStepAuthentication(AsyncTaskToken *token)
+{
+    QTextStream in(stdin);
+
+    qWarning ("Two step authentication PIN required.");
+
+    QString strPIN;
+    cout << "Enter PIN:";
+    in >> strPIN;
+
+    token->inParams["user_pin"] = strPIN;
+
+    tfaRequired = true;
+
+    gvApi.resumeTFALogin (token);
+}//MainWindow::onTwoStepAuthentication
+
 void
 MainWindow::getContactsDone (bool bChanges, bool bOK)
 {
@@ -486,3 +554,65 @@ MainWindow::onCommandForClient(const QString &command)
         getOut ();
     }
 }//MainWindow::onCommandForClient
+
+void
+MainWindow::loadCookies()
+{
+    QList<QNetworkCookie> cookies;
+    QSettings settings (strIni, QSettings::IniFormat, this);
+    QByteArray name, value;
+    quint32 expiration;
+    int i;
+
+    int size = settings.beginReadArray ("cookies");
+    for (i = 0; i < size; i++) {
+        name       = settings.value("name").toByteArray();
+        value      = settings.value("value").toByteArray();
+        expiration = settings.value("expiration").toUInt();
+
+        QNetworkCookie cookie(name, value);
+        cookie.setDomain(settings.value("domain").toString());
+
+        if (settings.value("isSession").toBool()) {
+            cookie.setExpirationDate (QDateTime::fromTime_t (expiration));
+        }
+
+        cookie.setHttpOnly(settings.value("isHttpOnly").toBool());
+        cookie.setSecure(settings.value("isSecure").toBool());
+        cookie.setPath(settings.value("path").toString());
+
+        cookies.append (cookie);
+    }
+    settings.endArray ();
+
+    gvApi.setAllCookies (cookies);
+
+    Q_DEBUG(QString("Loaded %1 cookies").arg(i));
+}//MainWindow::loadCookies
+
+void
+MainWindow::saveCookies()
+{
+    QList<QNetworkCookie> cookies = gvApi.getAllCookies ();
+    QSettings settings (strIni, QSettings::IniFormat, this);
+    int i;
+
+    settings.remove ("cookies");
+    settings.beginWriteArray ("cookies");
+
+    for (i = 0; i < cookies.count(); i++) {
+        settings.setArrayIndex (i);
+        settings.setValue ("name",       cookies.at(i).name());
+        settings.setValue ("value",      cookies.at(i).value());
+        settings.setValue ("expiration", cookies.at(i).expirationDate().toTime_t());
+        settings.setValue ("domain",     cookies.at(i).domain());
+        settings.setValue ("isSession",  cookies.at(i).isSessionCookie());
+        settings.setValue ("isSecure",   cookies.at(i).isSecure());
+        settings.setValue ("isHttpOnly", cookies.at(i).isHttpOnly());
+        settings.setValue ("path",       cookies.at(i).path());
+    }
+
+    settings.endArray ();
+
+    Q_DEBUG(QString("Saved %1 cookies").arg(i));
+}//MainWindow::saveCookies
